from pathlib import Path
from cli import pyke
import requests
import click
import time
import json
import sys
import os

class ComponentSetVersionBehavior:
    def __init__(self, profile=None, component_set=None, format=None, filter=None, page=None, pagesize=None, json=False):
        self.util = pyke.Util(profile=profile)
        if profile is not None:
            self.context = pyke.auth.load_cache()
        self.object_type = 'component-set'
        self.json = json
        self.profile = profile
        self.component_set = component_set
        self.format = format
        self.filter = filter
        self.page = page
        self.pagesize = pagesize

    def list(self, component_set):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)
        data = {'filter': self.filter, 'page': self.page, 'pagesize': self.pagesize}

        titles = {
          'directIncludes': '# Inc',
          'directCssIncludes': '# Css Inc',
          'expand__experiences': '# Exp',
          'expand__components': '# Comp',
        }

        handlers = {
          'directIncludes': lambda x: str(len(x)),
          'directCssIncludes': lambda x: str(len(x)),
          'expand__experiences': lambda x: str(len(x)),
          'expand__components': lambda x: str(len(x))
        }

        resp = self.util.cli_request('GET',
            self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions?expand=experiences&page={page}&pagesize={pagesize}&{filter}',
            {'component_set_id': component_set_id, **data}))

        if not self.json:
            self.util.print_table(resp['payload']['data'], self.format, titles=titles, handlers=handlers)
            self.util.print_record_count(resp)
        else:
            click.echo(json.dumps(resp))

    def list_components(self, component_set, version):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)
        version_id = self.util.get_version_id(self.object_type, component_set_id, version)

        resp = self.util.cli_request('GET',
            self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}?expand=components',\
            {'component_set_id': component_set_id, 'version_id': version_id}))

        click.echo(json.dumps(resp))

    def create(self, component_set, component_set_file_path, from_version, version_number,
        version_bump, css_includes, direct_includes, label):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)

        if from_version != '' and from_version is not None:
            try:
                from_version_id = int(from_version)
                version = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{from_version_id}',\
                    {'component_set_id': component_set_id, 'from_version_id': from_version_id} ))['payload']['data']
            except:
                version = self.util.get_version(self.object_type, component_set_id, from_version)

            if version is None:
                raise click.BadParameter(click.style("Component Set version not found", fg='red'), param_hint="--version-number")

            version[version_bump] = version[version_bump] + 1
            if version_bump in ['major', 'minor']:
                version['patch'] = 0
            if version_bump == 'major':
                version['minor'] = 0

        else:
            version = {}
            if label is None or label == '':
                label = input("Label: ")
                if label is None or label == '':
                    raise click.BadParameter(click.style("If you don't create from an old version then you must provide a --label", fg='red'), param_hint="--label")

            if version_number is None or version_number == '':
                version_number = input("Version Number: ")
                if version_number is None or version_number == '':
                    raise click.BadParameter(click.style("If you don't create from an old version then you must provide a --version-number in the format\
                        '<major: int>.<minor: int>.<patch: int>' ", fg='red'), param_hint="--version-number")

            major, minor, patch = self.util.parse_version_number(version_number)

            version['major'] = major
            version['minor'] = minor
            version['patch'] = patch

        if not self.json:
            click.echo('Image Size: {}'.format(self.util.get_size(component_set_file_path, image=False)))
            click.echo("Uploading Component Set Version to Lumavate")

        version['ephemeralKey'] = self.util.upload_ephemeral(component_set_file_path, 'application/zip')

        if label != '' and label is not None:
            version['label'] = label

        if css_includes:
            if 'directCssIncludes' not in version.keys():
                version['directCssIncludes'] = [x for x in css_includes]
            else:
                version['directCssIncludes'].extend([x for x in css_includes])

        if direct_includes:
            if 'directIncludes' not in version.keys():
                version['directIncludes'] = [x for x in direct_includes]
            else:
                version['directIncludes'].extend([x for x in direct_includes])

        resp = self.util.cli_request('POST',
            self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions',
            {'component_set_id': component_set_id}), data=json.dumps(version))

        if self.json:
            click.echo(json.dumps(resp))
            return

        handlers = {
          'directIncludes': lambda x: str(len(x)),
          'directCssIncludes': lambda x: str(len(x))
        }

        self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

    def update(self, component_set, version, label):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)
        version_id = self.util.get_version_id(self.object_type, component_set_id, version)

        post_data = {
          'componentSetId': component_set_id,
          'id': version_id,
          'label': label
        }

        resp = self.util.cli_request('PUT',
          self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}',
          {'component_set_id': component_set_id, 'version_id': version_id}), data=json.dumps(post_data))

        if self.json:
            click.echo(json.dumps(resp))
            return

        handlers = {
          'directIncludes': lambda x: str(len(x)),
          'directCssIncludes': lambda x: str(len(x))
        }

        self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

    def delete(self, component_set, version, version_mask):
        component_set_id = self.util.lookup_object_id(self.object_type, component_set)

        if version is None and version_mask is None:
            version = str(input("Version: "))
            if version is None or version == '':
                raise click.BadParameter(click.style('You must provide either a --version or a --version-mask', fg='red'))

        if version is not None:
            version_id = self.util.get_version_id(self.object_type, component_set_id, version)

            resp = self.util.cli_request('DELETE',
              self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}',
                {'component_set_id': component_set_id, 'version_id': version_id}))

            if self.json:
                click.echo(json.dumps(resp))
                return

            handlers = {
              'directIncludes': lambda x: str(len(x)),
              'directCssIncludes': lambda x: str(len(x))
            }

            self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

        else:
            versions = self.util.get_versions_from_mask(self.object_type, component_set_id, version_mask)
            if self.json:
                resp_list = {}
                resp_list['responses'] = []
                for v in versions:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}', {'component_set_id': component_set_id, 'version_id': v['id']}))
                    resp_list['responses'].append(resp)
                click.echo(json.dumps(resp_list))
                return

            with click.progressbar(versions, label='Deleting Versions') as bar:
                for v in bar:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/component-sets/{component_set_id}/versions/{version_id}', {'component_set_id': component_set_id, 'version_id': v['id']}))

            handlers = {
                  'directIncludes': lambda x: str(len(x)),
                  'directCssIncludes': lambda x: str(len(x))
            }

            self.util.print_table(versions, self.format, handlers=handlers)



from .microservice import MicroserviceBehavior
from .microservice_version import MicroserviceVersionBehavior
from .widget import WidgetBehavior
from .widget_version import WidgetVersionBehavior
from .component_set import ComponentSetBehavior
from .component_set_version import ComponentSetVersionBehavior



from functools import wraps
from pathlib import Path
from pyparsing import *
import dateutil.parser
import requests
import click
import copy
import json
import os


class auth:

    def get_profile(config_data, name):
        return config_data['profiles'].get(name)

    def get_env(config_data, name):
        return config_data['envs'].get(name)

    def set_cache(data):
        clean_data = auth.sanatize_data(data)
        with open(auth.get_cache_file(), 'w+') as outfile:
            json.dump(data, outfile, indent=4, sort_keys=True)

    def clear_cache():
        with open(auth.get_cache_file(), 'w+') as outfile:
            json.dump({}, outfile, indent=4, sort_keys=True)

    def set_cache_token(token):
        cache = auth.load_cache()
        cache['accessToken'] = token
        auth.set_cache(cache)

        return cache

    def get_profile_names(ctx, args, incomplete):
        config = auth.load_config()
        names = config['profiles'].keys()
        return [k for k in names if incomplete in k]

    def sanatize_data(data):
        return { k:v.replace(" ", "") if isinstance(v, str) and k != 'orgName' else v for k,v in data.items() }

    def set_profile_token(profile, token):
        config_data = auth.load_config()
        config_data['profiles'][profile]['accessToken'] = token
        auth.save_config(config_data)

        return config_data

    def get_cache_file():
        cache_path = str(Path.home()) + '/.luma_cache'
        if not os.path.exists(cache_path):
            with open(cache_path, 'w+') as cache:
                json.dump({}, cache)

        return cache_path

    def load_cache():
        with open(auth.get_cache_file()) as json_file:
            data = json.load(json_file)

        return data

    def save_config(data):
        with open(auth.get_config_file(), 'w+') as outfile:
            json.dump(data, outfile, indent=4, sort_keys=True)

    def load_config():
        try:
            with open(auth.get_config_file()) as json_file:
                data = json.load(json_file)
            return data
        except Exception as e:
            print('No config file or bad config file found. Making a new one.')
            auth.save_config({ "envs": {}, "profiles": {} })

            return {
                "envs": {},
                "profiles": {}
            }

    def get_config_file():
        config_path = str(Path.home()) + '/.luma_cli_config'
        if not os.path.exists(config_path):
            with open(config_path, 'w') as config:
                json.dump({ "envs": {}, "profiles": {} }, config)

        return config_path

    def profile_context_required(f):
        @wraps(f)
        def decorated_func(*args, **kwargs):
            try:
                profile = kwargs.get('profile')
                config_data = auth.load_config()
                profile_data = auth.get_profile(config_data, profile)

                if profile_data is None:
                    raise click.BadParameter(click.style('You must provide the name of a profile that you have configured', fg='red'))
                if profile_data.get('orgId') is None or profile_data.get('orgId') == '':
                    raise click.BadParameter(click.style('You must finish configuring this profile before you can use it', fg='red'))

                clean_data = auth.sanatize_data(profile_data)

                clean_data['profileName'] = profile
                env_data = auth.sanatize_data(auth.get_env(config_data, clean_data.get('env')))

                cache_data = {**clean_data, **env_data}

                # Protect against error state where 'accessToken' would become null
                if cache_data.get("accessToken") is None:
                    cache_data['accessToken'] = " "

                auth.set_cache(cache_data)

            except Exception as e:
                click.echo(click.style(str(e), fg='red'))
                raise click.Abort()

            f(*args, **kwargs)
            auth.clear_cache()

            return
        return decorated_func



import click


def validate_str_literal(type, value):
    if isinstance(type, list):
        return validate_list(value)

def validate_list(value):
    if value[0] != "[" and value[1] != "'"\
    and value[-2] != "'" and value[-1] != "]":
        raise click.BadParameter(css_includes)

def print_me():
    print("Hello World!!")


from pathlib import Path
from cli import pyke
import requests
import click
import json
import sys
import os

@click.group(help="Profiles add company context to commands. Almost all calls to Lumavate require context. \
Profiles use Envs to get tokens and then give those tokens context using the Org ID.")
def profile():
    pass

@click.command('ls', help="List the profiles you have configured.")
@click.option('--format', '-f', default='{profileName} {env} {orgName} {orgId}', help='Format')
@click.option('--json', 'json_flag', is_flag=True, help='Return raw json.')
def list(format, json_flag):
    config = pyke.auth.load_config()

    data = []
    for key in config['profiles'].keys():
        config['profiles'][key]['profileName'] = str(key)
        data.append(config['profiles'][key])

    if json_flag:
        click.echo(json.dumps(config))
        return

    pyke.Util().print_table(data, format)

@click.command()
@click.option('--profile-name', prompt=True, help='Profile Name')
@click.option('--env', default=None, help='The name of the Env you want to use with this profle')
@click.option('--format', '-f', default='{env} {orgName} {orgId}', help='The available column names for this command are {env} {orgName} {orgId}')
def add(profile_name, env, format):
    if not profile_name.strip():
        raise click.BadParameter(click.style("The --profile-name must contain more than just whitespace.", fg='red'))

    click.echo(' ')
    config = pyke.auth.load_config()

    titles = {
        'envName': 'Env Name',
        'app': 'App',
        'audience': 'Audience',
        'token': 'Token'
    }

    headers = '{envName} {app} {audience} {token}'

    if env is None:
        data = []
        for key in config['envs'].keys():
            config['envs'][key]['envName'] = str(key)
            data.append(config['envs'][key])

        pyke.Util().print_table(data, headers, titles=titles)
        click.echo(' ')

        env = input("Name of the Env you want to use with this profle: ")
        click.echo(' ')

    env_names = config["envs"].keys()
    if env not in env_names:
        raise click.BadParameter(click.style("You must select and Env that you have configured", fg='red'))

    config['profiles'][profile_name] = {
                                        "accessToken": " ",
                                        "env": env,
                                        "orgId": None,
                                        "orgName": ""
                                    }

    profile_data = config['profiles'][profile_name]

    resp = pyke.Util().list_companies(env)
    click.echo(' ')

    org_id = input("Org ID you want to associate with this profile: ")
    click.echo(' ')

    try:
        org_id = int(org_id)
    except Exception as e:
        raise click.BadParameter(click.style('You must provide the id of the organization', fg='red'))

    if str(org_id) not in [rec.get('id').replace(" ", "") for rec in resp]:
        raise click.BadParameter(click.style('You must provide the id of the organization you have access to', fg='red'))

    org_name = next(rec.get('name') for rec in resp if int(rec.get('id')) == org_id)

    profile_data["orgId"] = org_id
    profile_data["orgName"] = org_name.rstrip()

    config['profiles'][profile_name] = profile_data

    pyke.auth.save_config(config)

    profile_data["profile"] = profile_name

    titles = {
        'profile': 'Profile',
        'env': 'Environment',
        'orgName': 'Org Name',
        'orgId': 'Org ID'
    }

    pyke.Util().print_table([profile_data], format, titles=titles)
    click.echo(' ')

@click.command('rm')
@click.option('--profile', '-p', prompt=True, help='Profile to delete')
def delete(profile):
    config_data = pyke.auth.load_config()
    profile_data = config_data["profiles"][profile]
    del config_data["profiles"][profile]

    pyke.auth.save_config(config_data)

    click.echo("Deleted profile:")
    click.echo(json.dumps(profile_data))


profile.add_command(list)
profile.add_command(delete)
profile.add_command(add)



from cli import pyke, behavior
import click

@click.group('widget-version', help="A widget-version is the docker image of widget, \
along with the details it needs to run such as, port number and Env variables.")
def widget_version():
    pass

@click.command('ls')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--format', '-f', default='{id} {actualState} {versionNumber} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--filter', default='')
@click.option('--page', default=1)
@click.option('--pagesize', default=100)
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def list(widget, profile, format, json, **kwargs):
    behavior.WidgetVersionBehavior(profile=profile, format=format, json=json, **kwargs).list(widget)

@click.command('start')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@click.option('--format', '-f', default='{id} {actualState} {versionNumber} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def start(widget, version, profile, format, json):
    behavior.WidgetVersionBehavior(profile=profile, format=format, json=json).start_stop("start", widget, version)

@click.command('stop')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@click.option('--format', '-f', default='{id} {actualState} {versionNumber} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def stop(widget, version, profile, format, json):
    behavior.WidgetVersionBehavior(profile=profile, format=format, json=json).start_stop("stop", widget, version)

@click.command('exec', help='Pass in a command to be run directly on the docker container.')
@click.argument('command')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@click.option('--target', default='one', help="Run this command on ONE replica of the service or ALL replicas.")
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def run(command, widget, version, profile, target, json):
    behavior.WidgetVersionBehavior(profile=profile, json=json).run(widget, version, command, target)

@click.command('logs')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def logs(widget, version, profile, json):
    behavior.WidgetVersionBehavior(profile=profile, json=json).logs(widget, version)

@click.command('add')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--port', type=int, help='The widgets port number')
@click.option('--widget', '-w', prompt=True)
@click.option('--widget-file-path', '-path', type=click.Path(), help='The image must be a gzipped tar file. Ex: {file_name}.tar.gz')
@click.option('--docker-image', '-image', type=str, help='The name (including the tag) of a docker image from the docker daemon. You must have docker running locally to use this option.')
@click.option('--from-version', '-fv', type=str, help="Take all values from an exsisting version and create a new version. \
Increment the major, minor or patch version by 1 with --major, --minor, --patch flags. Override any old values by passing them in as options.")
@click.option('--version', '-v', type=str, default=None, help="The version number of the new widget version")
@click.option('--patch', 'version_bump', flag_value='patch', default=True)
@click.option('--minor', 'version_bump', flag_value='minor')
@click.option('--major', 'version_bump', flag_value='major')
@click.option('--env-var', 'env', default=None, multiple=True, help="The environment variables to add to docker image when it is run. It must be valid json wrapped in single quotes.")
@click.option('--label', '-l', type=click.Choice(['prod', 'dev', 'old']))
@click.option('--format', '-f', default='{id} {actualState} {versionNumber} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def create(widget, profile, port, widget_file_path,\
    docker_image, from_version, version, version_bump, env, label, format, json):
    behavior.WidgetVersionBehavior(profile=profile, json=json).create(widget, port,\
        widget_file_path, docker_image, from_version, version, version_bump, env, label, format)

@click.command()
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--version', '-v', prompt=True, help='The version ID or the version-number')
@click.option('--label', '-l', prompt=True)
@click.option('--format', '-f', default='{id} {versionNumber} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def update(widget, version, label, profile, format, json):
    behavior.WidgetVersionBehavior(profile=profile, json=json).update(widget, version, label, format)

@click.command('rm')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command', autocompletion=pyke.auth.get_profile_names)
@click.option('--widget', '-w', prompt=True)
@click.option('--version-mask', '-vm', type=str, help="Delete versions with version numbers that match this version mask")
@click.option('--version', '-v', help='The version ID or the version-number')
@click.option('--format', '-f', default='{id} {versionNumber} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False)
@pyke.auth.profile_context_required
def delete(widget, version_mask, version, profile, format, json):
    behavior.WidgetVersionBehavior(profile=profile, json=json).delete(widget, version, version_mask, format)

widget_version.add_command(list)
widget_version.add_command(create)
widget_version.add_command(start)
widget_version.add_command(stop)
widget_version.add_command(delete)
widget_version.add_command(update)
widget_version.add_command(run)
widget_version.add_command(logs)
